ERAS_HAS_RUN_INITIAL_CONFIG_VAR_NAME = "ERAS_HAS_RUN_INITIAL_CONFIG"
ERAS_OPENAI_KEY_ENV_VAR_NAME = "ERAS_OPENAI_KEY"
ERAS_BASE_URL_ENV_VAR_NAME = "ERAS_BASE_URL"
ERAS_MODEL_ENV_VAR_NAME = "ERAS_MODEL"
OPENAI_MODEL_DEFAULT = "gpt-4"